#
# LibreQuake Map Builder Python Script
# v0.0.3
# ---
# MissLavander-LQ & cypress/MotoLegacy & ZungryWare/ZungrySoft
#
import os
import re
import subprocess
import sys
import shutil

# Map compiler paths
LQ_BSP_PATH = "qbsp"
LQ_VIS_PATH = "vis"
LQ_LIG_PATH = "light"

# Location of per-map compilation argument paths
LQ_MAP_CON_PATH = "src/-buildconfigs-"

# Default compilation flags
LQ_DEF_BSP_FLAGS = ""
LQ_DEF_VIS_FLAGS = ""
LQ_DEF_LIG_FLAGS = "-bounce -dirt"

LQ_GLOBAL_BSP_FLAGS = "-bspx"
LQ_GLOBAL_VIS_FLAGS = ""
LQ_GLOBAL_LIG_FLAGS = ""


# Check for compiler
def check_for_compiler():
    if not LQ_BSP_PATH:
        if not shutil.which('qbsp'):
            print("Couldn't find ericw-tools, required to build map content.")
            print("Please see https://github.com/ericwa/ericw-tools/ for info.")
            sys.exit()


def get_config_path(map_path):
    # We know that the config dir mirrors the map dir structure so we need to remove the common path.
    # whats left is the relative path to the map file.
    # Then we exchange the extension from .map to .conf
    map_abs_path = os.path.abspath(map_path)
    conf_abs_path = os.path.abspath(LQ_MAP_CON_PATH)
    common_path = os.path.commonpath([map_abs_path, conf_abs_path])
    map_rel_path = os.path.relpath(map_abs_path, common_path)
    map_rel_path_no_ext = os.path.splitext(map_rel_path)[0]
    return os.path.join(LQ_MAP_CON_PATH, f"{map_rel_path_no_ext}.conf")


def get_config(config_path):
    # Read the config file and return a dictionary of key-value pairs
    if not os.path.exists(config_path):
        print(f"No config file found at {config_path}. Using default values.")
        return {}

    config = {}
    with open(config_path, "r") as config_file:
        for line in config_file:
            # Strip whitespace and ignore empty lines or comments
            line = line.strip()
            if line and not line.startswith("#"):
                key, value = line.split("=", 1)  # Split on the first '=' only
                config[key.strip()] = value.strip('"').strip()
    return config


def get_compile_flags(map_path):
    config_path = get_config_path(map_path)
    config = get_config(config_path)

    LQ_BSP_FLAGS = config.get("LQ_BSP_FLAGS", LQ_DEF_BSP_FLAGS) + " " + LQ_GLOBAL_BSP_FLAGS
    LQ_VIS_FLAGS = config.get("LQ_VIS_FLAGS", LQ_DEF_VIS_FLAGS) + " " + LQ_GLOBAL_VIS_FLAGS
    LQ_LIG_FLAGS = config.get("LQ_LIG_FLAGS", LQ_DEF_LIG_FLAGS) + " " + LQ_GLOBAL_LIG_FLAGS

    return LQ_BSP_FLAGS, LQ_VIS_FLAGS, LQ_LIG_FLAGS


# Execute command
def execute_command(command, fail_on_regexes=[], **kwargs):
    try:
        result = subprocess.run(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            check=True,  # Fail on non-zero exit code
            **kwargs,
        )
    except subprocess.CalledProcessError as e:
        print(f"!!! Command failed:\n{e.stdout.decode('utf-8')}")
        raise e

    stdout_str = result.stdout.decode("utf-8")
    for pattern in fail_on_regexes:
        if re.search(pattern, stdout_str):
            print(stdout_str)
            raise ValueError(f"!!! Command failed: `{pattern}` found in output")


# Find all files in directory with this extension
def find(directory, ext):
    result_list = []
    for root, dirs, files in os.walk(directory):
        for file in files:
            if file.endswith(ext):
                result_list.append(os.path.join(root, file))
    return result_list


def move_file(src, dest_dir):
    # Get the base filename
    filename = os.path.basename(src)

    # Construct the full path for the destination file
    dest_file = os.path.join(dest_dir, filename)

    # Check if the destination file exists and remove it if it does
    if os.path.isfile(dest_file):
        os.remove(dest_file)

    # Move the source file to the destination directory
    shutil.move(src, dest_file)


# Command make
def command_make(specific_map=None, specific_dir=None):
    check_for_compiler()

    search_dir = specific_dir if specific_dir else './'

    for f in find(search_dir, '.map'):
        map_name = os.path.splitext(os.path.basename(f))[0]

        if specific_map and map_name != specific_map:
            continue

        if not specific_map:
            if "skip" in f:
                continue
            if "archive" in f:
                continue
            if "autosave" in f:
                continue

        LQ_BSP_FLAGS, LQ_VIS_FLAGS, LQ_LIG_FLAGS = get_compile_flags(f)

        print(f"Compiling {f}")

        unwanted_bsp_warnings = [
            r"WARNING: unable to (find|load) texture",
            r"WARNING: Couldn't locate texture",
        ]
        execute_command(
            [LQ_BSP_PATH] + LQ_BSP_FLAGS.split() + [f"{map_name}.map"], cwd=os.path.dirname(f),
            fail_on_regexes=unwanted_bsp_warnings,
        )

        if "LQ_SKIP" not in LQ_VIS_FLAGS:
            execute_command([LQ_VIS_PATH] + LQ_VIS_FLAGS.split() + [f"{map_name}.bsp"], cwd=os.path.dirname(f))

        if "LQ_SKIP" not in LQ_LIG_FLAGS:
            execute_command([LQ_LIG_PATH] + LQ_LIG_FLAGS.split() + [f"{map_name}.bsp"], cwd=os.path.dirname(f))

    # Move bsp and lit files into the /lq1/maps directory
    print("Moving files...")
    for ext in [".bsp", ".lit"]:
        for file in find('./src', ext):
            move_file(file, "./")

    print("* Build DONE")


# Command single
def command_single(map_name):
    if not map_name:
        print("Map name needed, e.g. 'lq_e1m1'")
        return
    command_make(specific_map=map_name)


# Command directory
def command_directory(directory):
    if not directory:
        print("Directory name needed, e.g. 'e1'")
        return
    command_make(specific_dir=directory)


# Command clean
def command_clean():
    for ext in [".bsp", ".lit", ".prt", ".texinfo", ".texinfo.json", ".pts", ".log"]:
        for file in find('./', ext):
            os.remove(file)
    print("* Clean DONE")


# Command help
def command_help():
    help_text = """
                 LibreQuake make.py Map Builder Script /// v0.0.3
    ==========================================================================
    This script requires ericw-tools (https://github.com/ericwa/ericw-tools/).
    --------------------------------------------------------------------------
    Usage: make.py [OPERATION] [FLAGS]

    OPERATIONS:
    -h, help            read this msg :3
    -c, clean           remove extra files that are left after build
    -m, make            compile all available map OR
    -s, single <MAP>    compile a specified single map
    -d, directory <DIR> compile all maps in a specified directory

    FLAGS:
    QBSP_FLAGS    override map-specific qbsp flags with a global setting
    QBSP_PATH     use a local path for qbsp instead of checking $PATH
    VIS_FLAGS     override map-specific vis flags with a global setting
    VIS_PATH     use a local path for qbsp instead of checking $PATH
    LIGHT_FLAGS   override map-specific light flags with a global setting
    LIGHT_PATH     use a local path for qbsp instead of checking $PATH

    Example: make.py -m LIGHT_FLAGS=\"-extra4 -bounce\" QBSP_PATH=\"~/qbsp\"
    Example: make.py -s e1/lq_e1m1
    ==========================================================================
    """
    print(help_text)


# Argument parsing
def parse_args(args):
    operations = {
        '-m': command_make,
        '-s': lambda: command_single(args[1] if len(args) > 1 else None),
        '-d': lambda: command_directory(args[1] if len(args) > 1 else None),
        '-c': command_clean,
        '-h': command_help,
    }

    operation = operations.get(args[0], command_help)
    operation()


# If being run from the command line
if __name__ == "__main__":
    if (len(sys.argv) > 1):
        parse_args(sys.argv[1:])
    else:
        command_help()

# If being run from build.py
if __name__ == "__build__":
    command_make()



# Code written by ZungryWare

# This script copies the necessary files to build the different releases. The
# releases are specified in build_releases.json and they specify which
# components to use. Components are specified in build_components.json and each
# one is a big list of files that a release either wants all of or none of. Each
# component also specifies whether each file should end up in pak0.pak,
# pak1.pak, or just in the root folder without being packed. Files are copied to
# the 'working' directory, compiled, then copied over to a folder in the
# 'releases' directory. Before doing this, the script also compiles all of the
# wad and bsp/lit files so they can be copied over.

import subprocess
import os
import json
import shutil
import runpy
import glob
import argparse


# Builds a single file
def build_file(source_path, destination_path, source_if_missing):
    os.makedirs(os.path.dirname(destination_path), exist_ok=True)
    try:
        shutil.copy(source_path, destination_path)
    except Exception:
        if len(source_if_missing):
            shutil.copy(source_if_missing, destination_path)
        else:
            # No error for lit files since not all maps have them
            if not destination_path.endswith(".lit"):
                raise ValueError(f"!!! Error: Missing file with no substitute: {source_path}")


# Gets a source and dest path from a file entry File. Entries can be a string
# (where source and dest are the same) or a 2-long list (specifying the source
# and dest, respectively)
def extract_file(file):
    if type(file) is list:
        return file[0], file[1]
    else:
        return file, file


# Builds a single component from build_components.json
def build_component(component_data):
    # Determine the source if missing
    # This is the file to use as a substitute if the desired file is missing
    if component_data['source_if_missing']:
        source_if_missing = os.path.join('lq1/', component_data['source_if_missing'])
    else:
        source_if_missing = ""

    base_dir = component_data['base_dir']

    for file in component_data['files_pak0']:
        file_source, file_dest = extract_file(file)
        build_file(
            os.path.join(base_dir, file_source),
            os.path.join('working/pak0/', file_dest),
            source_if_missing,
        )
    for file in component_data['files_pak1']:
        file_source, file_dest = extract_file(file)
        build_file(
            os.path.join(base_dir, file_source),
            os.path.join('working/pak1/', file_dest),
            source_if_missing,
        )
    for file in component_data['files_unpacked']:
        file_source, file_dest = extract_file(file)
        build_file(
            os.path.join(base_dir, file_source),
            os.path.join('working/unpacked/', file_dest),
            source_if_missing,
        )


# Build a single release from build_releases.json
def build_release(name, data):
    # Print
    print(f"Building release {name}...")

    # Clear working and set up working directories
    clear_working()

    # Create release directory
    os.makedirs('releases', exist_ok=True)

    # Pull out base directory
    base_dir = data['base_dir']

    # Build components in release
    with open('build_components.json', 'r') as json_file:
        components = json.load(json_file)
        for component_name in data['components']:
            print(f"Building component {component_name}...")
            build_component(components[component_name])

    # Copy stuff over to release folder
    shutil.copytree('working/unpacked', os.path.join('releases', name, base_dir))

    # Build and copy pak0
    pak0_exists = len(os.listdir('working/pak0')) > 0
    if pak0_exists:
        os.chdir('working/pak0')
        filepaths = glob.glob('*')
        command = ['qpakman']
        command.extend(filepaths)
        command.extend(['-o', '../pak0.pak'])
        print("Creating pak0.pak...")
        try:
            subprocess.run(
                command,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                check=True,  # Fail on non-zero exit code
            )
        except subprocess.CalledProcessError as e:
            print(f"!!! Command failed:\n{e.stdout.decode('utf-8')}")
            raise e
        os.chdir('../../')
        shutil.copy('working/pak0.pak', os.path.join('releases', name, base_dir, 'pak0.pak'))

    # Build and copy pak1
    pak1_exists = len(os.listdir('working/pak1')) > 0
    if pak1_exists:
        os.chdir('working/pak1')
        filepaths = glob.glob('*')
        command = ['qpakman']
        command.extend(filepaths)
        command.extend(['-o', '../pak1.pak'])
        print('Creating pak1.pak...')
        try:
            subprocess.run(
                command,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                check=True  # Fail on non-zero exit code
            )
        except subprocess.CalledProcessError as e:
            print(f"!!! Command failed:\n{e.stdout.decode('utf-8')}")
            raise e
        os.chdir('../../')
        shutil.copy('working/pak1.pak', os.path.join('releases', name, base_dir, 'pak1.pak'))


# Clears the working directory and sets up empty directories
def clear_working():
    # Remove working folder and its contents
    shutil.rmtree('./working', ignore_errors=True)

    # Create new empty directories
    os.makedirs('working', exist_ok=True)
    os.makedirs('working/pak0', exist_ok=True)
    os.makedirs('working/pak1', exist_ok=True)
    os.makedirs('working/unpacked', exist_ok=True)


def compile_wad():
    os.chdir('texture-wads')
    runpy.run_path('./compile_wads.py', run_name="__build__")
    os.chdir('../')


def compile_bsp():
    os.chdir('./lq1/maps')
    runpy.run_path('./compile_maps.py', run_name="__build__")
    os.chdir('../../')


def compile_progs():
    try:
        print("Compiling progs.dat...")
        subprocess.run(
            ["fteqcc", "qcsrc/progs.src", "-D__LIBREQUAKE__", "-O3"],
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            check=True,  # Fail on non-zero exit code
        )

        print("Compiling qwprogs.dat...")
        subprocess.run(
            ["fteqcc", "qcsrc/progs.src", "-D__LIBREQUAKE__", "-D__QW__", "-O3"],
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            check=True,  # Fail on non-zero exit code
        )
    except subprocess.CalledProcessError as e:
        print(f"!!! Command failed:\n{e.stdout.decode('utf-8')}")
        raise e


def compile():
    compile_progs()
    compile_wad()
    compile_bsp()


def build():
    # Delete existing releases
    shutil.rmtree('./releases', ignore_errors=True)

    # Build each release one by one
    with open('build_releases.json', 'r') as json_file:
        releases = json.load(json_file)
        for key, value in releases.items():
            build_release(key, value)


def main():
    # Parse command-line arguments
    parser = argparse.ArgumentParser(description="Example script to demonstrate argparse usage.")
    parser.add_argument('-c', '--compile', action='store_true', help="compile assets without building releases")
    parser.add_argument('-w', '--compile-wads', action='store_true', help="just compile the texture wads")
    parser.add_argument('-m', '--compile-maps', action='store_true', help="just compile the maps")
    parser.add_argument('-p', '--compile-progs', action='store_true', help="just compile the progs.dat")
    parser.add_argument('-b', '--build', action='store_true', help="build releases without recompiling")
    args = parser.parse_args()

    # Conflicting flags error
    if args.build and (args.compile or args.compile_wads or args.compile_maps or args.compile_progs):
        print("Incompatible flags. Use -h for help.")
    # Compile specific flags
    elif args.compile_wads or args.compile_maps or args.compile_progs:
        if args.compile_wads:
            compile_wad()
        if args.compile_maps:
            compile_bsp()
        if args.compile_progs:
            compile_progs()
    # Compile all flag
    elif args.compile:
        compile()
    # No compile flag
    elif args.build:
        build()
    # Regular mode
    else:
        compile()
        build()

    # Confirmation
    print("Build complete!")


if __name__ == "__main__":
    main()



import glob


def check_player_starts(target_file: str) -> bool:
    # We care about Deathmatch starts in all types of maps
    min_starts = {
        "deathmatch": 4
    }
    if "src/dm" not in target_file:
        # For campaign maps only, we want to also check singleplayer and coop starts
        min_starts.update({
            "start": 1,
            "coop": 3
        })

    with open(target_file, "r") as map_file:
        map_data = map_file.read()

    for start_type, min_count in min_starts.items():
        starts_in_map = map_data.count(f"info_player_{start_type}")
        if starts_in_map < min_count:
            print(f"{target_file}:0:0: [min_{start_type}_starts] "
                  f"At least {min_count} {start_type} are required but only {starts_in_map} were found.")


if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("mapfile", nargs="*", default=glob.glob("lq1/maps/src/*/*.map"),
                        help="A list of .map files to read.")

    args = parser.parse_args()
    for filename in args.mapfile:
        check_player_starts(filename)



import subprocess
import glob


def make():
    wads_list = [
        'lq_dev',
        'lq_dev_legacy',
        'lq_flesh',
        'lq_greek',
        'lq_health_ammo',
        'lq_legacy',
        'lq_liquidsky',
        'lq_mayan',
        'lq_medieval',
        'lq_palette',
        'lq_metal',
        'lq_props',
        'lq_tech',
        'lq_terra',
        'lq_utility',
        'lq_wood',
        'lq_conc',
    ]
    for wad in wads_list:
        filepaths = glob.glob(f'{wad}/*')
        command = ['qpakman']
        command.extend(filepaths)
        command.extend(['-o', f'{wad}.wad'])
        print(f'Compiling {wad}.wad...')
        try:
            subprocess.run(
                command,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                check=True,  # Fail on non-zero exit code
            )
        except subprocess.CalledProcessError as e:
            print(f"!!! Command failed:\n{e.stdout.decode('utf-8')}")
            raise e


if __name__ == "__main__":
    make()

if __name__ == "__build__":
    make()




import os
from PIL import Image

base_folder = os.path.abspath(os.path.dirname(__file__)).replace("\\", "/")
converted_folder = base_folder+"/converted/"
if not os.path.exists(converted_folder):
    os.makedirs(converted_folder)

files = os.listdir(base_folder)

converted = 0
for name in files:
    try:
        img = Image.open(name)
    except (PermissionError, OSError):
        # skip anything that can't be opened or isn't an image
        continue

    output_file = converted_folder + name

    if not img.width % 16 == 0:
        new_width = img.width - (img.width % 16) + 16

    else:
        new_width = img.width

    if not img.height % 16 == 0:
        new_height = img.height - (img.height % 16) + 16
    else:
        new_height = img.height

    print(f"Converting: ({img.width}x{img.height}) -> ({new_width}x{new_height}) - {output_file}")

    resized = img.resize((new_width, new_height), Image.NEAREST)
    resized.save(output_file)
    converted += 1

plural = "" if converted == 1 else "s"
print(f"Total: {converted} image{plural} converted.")



'''
A script that analyzes the LibreQuake asset list etherpad
Lists out what assets still are unaccounted for, what assets are
accounted for, and who has claimed them.
Then prints out the ratio of claimed assets to total assets.
Script written by ZungryWare
'''
import sys

console = 1
#Green text
if console == 1:
	col = '\033[92m'
	end = '\033[0m'
else:
	col = "**"
	end = "**"

def bold(str_in):
	return col + str(str_in) + end

def bprint(s):
	sys.stdout.write(s)

def main():
	print(bold("LibreQuake Asset report:\n"))
	
	count_total = 0
	count_complete = 0
	
	#Go through the file, listing incomplete items
	print(bold("Items that still need to be accounted for:"))
	file1 = open("LQ_Assets.txt","r")
	#Go through each line
	for line in file1:
		#Only read lines that contain a file
		if line[:3] == "pak" or line[:3] == "mus":
			#Track the total number of items
			count_total += 1
			items = line.split(" - ")
			#Ensure there is a name on the list
			if len(items) == 1:
				bprint(items[0])
			elif len(items) >= 2:
				if items[1] == "\n":
					bprint(items[0])
	file1.close()
	
	#Go through the file again, listing complete items
	print(bold("\nItems that are complete or in progress:"))
	
	file1 = open("LQ_Assets.txt","r")
	for line in file1:
		if line[:3] == "pak" or line[:3] == "mus":
			items = line.split(" - ")
			#Ensure there is not a name on the list
			if len(items) >= 2:
				if items[1] != "\n":
					#Track the total number of COMPLETED items
					count_complete += 1
					bprint(bold(items[0]) + " is being accounted for by " + bold(items[1]))
	file1.close()
	print("\nItems accounted for: " + bold(str(count_complete) + "/" + str(count_total)))
	percent = (1.0*count_complete/count_total)*100
	percent_truncate = int(percent * 1000) / 1000.0
	print("We're " + bold(str(percent_truncate) + "%") + " of the way there!")
	
main()
